PhysFrame

Struct PhysFrame 

pub struct PhysFrame<S: PageSize = Size4KiB> { /* private fields */ }
A physical memory frame.

Implementations

Source

impl<S: PageSize> PhysFrame<S>

pub const fn from_start_address( address: PhysAddr, ) -> Result<Self, AddressNotAligned>

Returns the frame that starts at the given virtual address.

Returns an error if the address is not correctly aligned (i.e. is not a valid frame start).

pub const unsafe fn from_start_address_unchecked( start_address: PhysAddr, ) -> Self

Returns the frame that starts at the given virtual address.

Safety

The address must be correctly aligned.

pub const fn from_pfn(pfn: u64) -> Self

Returns the frame by a physical frame number.

use x86_64::{PhysAddr, structures::paging::{PhysFrame, Size4KiB}};

assert_eq!(PhysFrame::<Size4KiB>::from_pfn(0x123), PhysFrame::<Size4KiB>::containing_address(PhysAddr::new(0x123000)));
Panics

This function will panic if the resulting address is not valid.

pub const fn try_from_pfn(pfn: u64) -> Result<Self, PfnNotValid>

Returns the frame by a physical frame number.

use x86_64::{PhysAddr, structures::paging::{PhysFrame, Size4KiB}};

assert_eq!(PhysFrame::<Size4KiB>::try_from_pfn(0x123), Ok(PhysFrame::<Size4KiB>::containing_address(PhysAddr::new(0x123000))));
Error

This function will return an error if the resulting address is not valid.

pub const unsafe fn from_pfn_unchecked(pfn: u64) -> Self

Returns the frame by a physical frame number.

Safety

The resulting address must be valid.

pub const fn containing_address(address: PhysAddr) -> Self

Returns the frame that contains the given physical address.

pub const fn start_address(self) -> PhysAddr

Returns the start address of the frame.

pub const fn size(self) -> u64

Returns the size the frame (4KB, 2MB or 1GB).

pub const fn pfn(self) -> u64

Returns the PFN of the current frame.

The PFN is defined to be the address divided by the page size.

use x86_64::{PhysAddr, structures::paging::{PhysFrame, Size1GiB, Size2MiB, Size4KiB}};

assert_eq!(PhysFrame::<Size4KiB>::containing_address(PhysAddr::new(0x123000)).pfn(), 0x123);

// Note that this means that the PFN for the same address will be
// different for different page sizes.
assert_eq!(PhysFrame::<Size4KiB>::containing_address(PhysAddr::new(0xC000_0000)).pfn(), 0xC0000);
assert_eq!(PhysFrame::<Size2MiB>::containing_address(PhysAddr::new(0xC000_0000)).pfn(), 0x600);
assert_eq!(PhysFrame::<Size1GiB>::containing_address(PhysAddr::new(0xC000_0000)).pfn(), 0x3);
pub const fn range(start: PhysFrame<S>, end: PhysFrame<S>) -> PhysFrameRange<S>

Returns a range of frames, exclusive end.

pub const fn range_inclusive( start: PhysFrame<S>, end: PhysFrame<S>, ) -> PhysFrameRangeInclusive<S>

Returns a range of frames, inclusive end.